all_states = {
	'Immobile',
	'Moving',
	'Attacking',
	'Guarding',
	'Building',
	'Upgrading',
	'WaitingForTransport',
	'TransportLoading',
	'TransportUnloading',
	'MovingDown',
	'MovingUp',
	'Patrolling',
	'Busy',
	'Attached',
	'BeingReclaimed',
	'Repairing',
	'Diving',
	'Surfacing',
	'Teleporting',
	'Ferrying',
	'WaitForFerry',
	'AssistMoving',
	'PathFinding',
	'ProblemGettingToGoal',
	'NeedToTerminateTask',
	'Capturing',
	'BeingCaptured',
	'Reclaiming',
	'AssistingCommander',
	'Refueling',
	'GuardBusy',
	'ForceSpeedThrough',
	'UnSelectable',
	'DoNotTarget',
	'LandingOnPlatform',
	'CannotFindPlaceToLand',
	'BeingUpgraded',
	'Enhancing',
	'BeingBuilt',
	'NoReclaim',
	'NoCost',
	'BlockCommandQueue',
	'MakingAttackRun',
	'HoldingPattern',
	'SiloBuildingAmmo',
}

lock_states = {
	'Moving',
	'Attacking',
	'Guarding',
	'Building',
	'WaitingForTransport',
	'TransportLoading',
	'TransportUnloading',
	'MovingDown',
	'MovingUp',
	'Patrolling',
	'BeingReclaimed',
	'Repairing',
	'Diving',
	'Surfacing',
	'Teleporting',
	'Ferrying',
	'WaitForFerry',
	'AssistMoving',
	'PathFinding',
	'Capturing',
	'Reclaiming',
	'AssistingCommander',
	'Refueling',
	'GuardBusy',
	'LandingOnPlatform',
	'CannotFindPlaceToLand',
	'Enhancing',
	'MakingAttackRun',
	'HoldingPattern'
}

debugAI = false

oldUnit = Unit
Unit = Class(oldUnit) {

	SetCustomName = function(self, name)
		-- Set the unit name, not just its label
		self.name = name
		oldUnit.SetCustomName(self, name)
	end,

	SetCustomLabel = function(self, name)
		-- Set the unit label only
		oldUnit.SetCustomName(self, name)
	end,
	
	CheckStateThread = function(self)
		while not self:IsDead() do
			if debugAI then
				local state = self:GetStateString()
				local newname = ''
				if self.name ~= '' and state ~= '' then
					newname = self.name..': '..state
				elseif self.name ~= '' then
					newname = self.name
				else
					newname = self:GetStateString()
				end
				oldUnit.SetCustomName(self, newname)
			end
			WaitSeconds(3.0)
		end
	end,
	
	OnCreate = function(self)
		oldUnit.OnCreate(self)
		self.name = ''
		if debugAI then
			self:ForkThread(self.CheckStateThread)
		end
	end,
	
	Unlock = function(self)
		self:SetCustomName('Unlocked')
		self.locked = false
	end,

	Lock = function(self, name)
		self:SetCustomName('Locked for '..name)
		self.locked = true
	end,

	GetLock = function(self)
		local lock = nil 
		if not self:IsDead() then 
			for _,stateName in lock_states do
				if self:IsUnitState(stateName) then
					lock = stateName
					break
				end
			end
		end
		return lock
	end,
	
	IsLocked = function(self)
		if self:GetLock() then
			return true
		else
			return false
		end
	end,

	GetStateString = function(self)
		-- Returns a comma-seperated list of current states or empty string if idle
		local name = nil
		for _,state in all_states do
			if self:IsUnitState(state) then
				if not name then
					name = state
				else
					name = name ..','.. state
				end
			end
		end
		return name or ''
	end,

	OnStartBuild = function(self,unitBeingBuilt, order)
		oldUnit.OnStartBuild(self,unitBeingBuilt, order)
		-- Restore work progress when building units
		if self.focusHealth then
			LOG('LOAD: '..repr(order))
			LOG('LOAD: Restoring health '..self.focusHealth..' to '..self.focusLabel)
			unitBeingBuilt:SetHealth(self, self.focusHealth)
			self.focusHealth = nil
		end
	end,

	RestoreMissileProgress = function(self, weapon)
		-- Thread that compares build progress with saved build progress because I can't set the progress
		-- local thread function to stop missile build when missile is 'built'
		local function StopMissileRestore(self)
			WaitSeconds(0.1)
			-- TODO: Work out how to only stop the build, not the whole unit
			IssueStop({self})
		end
		-- wait for build to complete or be cancelled
		while self.missileProgress and self:IsUnitState('SiloBuildingAmmo') do
			local progress = self:GetWorkProgress() + self.missileProgress
			if progress >= 1.0 then
				if EntityCategoryContains(categories.NUKE, self) then
					LOG('LOAD: Saved partial nuke completed')
					self:GiveNukeSiloAmmo(1)
				else
					LOG('LOAD: Saved partial tactical missile completed')
					self:GiveTacticalSiloAmmo(1)
				end
				self:ForkThread(StopMissileRestore)
				break
			end
			WaitSeconds(0.1)
		end
		LOG('Silo build cancelled')
		self.missileProgress = nil
	end,
	
	OnSiloBuildStart = function(self,weapon)
		oldUnit.OnSiloBuildStart(self,weapon)
		-- Restore work progress when building units
		if self.missileProgress then
			LOG('LOAD: Restoring missile progress from '..self.missileProgress)
			self:ForkThread(self.RestoreMissileProgress, weapon)
		end
	end,
}